from pillow_mbm.version import __version__

import struct
from PIL import Image, ImageFile

MAGIC = b'\x03KSP'


def _accept(prefix: bytes):
    """Check if a file is a MBM file"""
    return prefix[:4] == MAGIC


class MBMImageFile(ImageFile.ImageFile):
    format = 'MBM'
    format_description = 'Kerbal Space Program MBM image'

    def _open(self):
        """Open an MBM file"""
        magic = self.fp.read(4)
        if magic != MAGIC:
            raise SyntaxError('not a MBM file')

        width, height, bits = struct.unpack('<2I4xI', self.fp.read(16))

        self._size = (width, height)

        if bits == 24:
            self.mode = 'RGB'
        elif bits == 32:
            self.mode = 'RGBA'
        else:
            raise SyntaxError('unknown number of bits')

        self.tile = [('raw', (0, 0, width, height), 20, (self.mode, 0, 1))]


Image.register_open(MBMImageFile.format, MBMImageFile, _accept)
Image.register_extensions(MBMImageFile.format, ['.mbm'])



import pathlib
import os
import click
from typing import List
from PIL import Image
import pillow_mbm


def get_decoded_extensions(feature: str = 'open') -> List[str]:
    """Gets a list of extensions for Pillow formats supporting a supplied feature"""
    Image.init()
    extensions = Image.EXTENSION
    formats = getattr(Image, feature.upper()).keys()

    return [ext for ext, fmt in extensions.items() if fmt in formats]


decoded_extensions = get_decoded_extensions('save')


# noinspection PyUnusedLocal
def validate_decoded_extension(ctx, param, value) -> str:
    """Check if an extension for a decoded image is valid"""
    if value[0] != '.':
        value = '.' + value

    if value not in decoded_extensions:
        raise click.BadParameter(f'Invalid extension for decoded file. Valid extensions are:\n{decoded_extensions}')

    return value


def path_pairs(inputs, output, suffix, extension):
    if len(inputs) < 1:
        raise click.BadArgumentUsage('No input files were provided.')

    inpaths = [pathlib.Path(i) for i in inputs]

    if not output:
        # decode in place
        return [(inpath, inpath.with_name(inpath.stem + suffix + extension)) for inpath in inpaths]

    else:
        outpath = pathlib.Path(output)
        if outpath.is_file():
            # decode to a file
            if len(inputs) > 1:
                raise click.BadOptionUsage('output', 'Output is a single file, but multiple input files were provided.')
            if outpath.suffix not in decoded_extensions:
                raise click.BadOptionUsage('output', f'File has incorrect extension for decoded file. Valid extensions are:\n{decoded_extensions}')

            return [(inpath, outpath) for inpath in inpaths]
        else:
            # decode to directory
            return [(inpath, outpath / (inpath.stem + suffix + extension)) for inpath in inpaths]


@click.command()
@click.option('-f/-F', '--flip/--no-flip', default=True, show_default=False, help="Vertically flip image after converting.")
@click.option('-r', '--remove', is_flag=True, help="Remove input images after converting.")
@click.option('-s', '--suffix', type=str, default='', help="Suffix to append to output file(s). Ignored if output is a single file.")
@click.option('-x', '--extension',
              callback=validate_decoded_extension,
              type=str, default='.png', show_default=True,
              help="Extension to use for output. Ignored if output is a single file. Output filetype is deduced from this")
@click.option('-o', '--output',
              type=click.Path(writable=True), default=None,
              help="Output file or directory. If outputting to a file, input filenames must be only a single item. By default, files are decoded in place.")
@click.option('-v', '--verbose', is_flag=True, help="print more information")
@click.argument('filenames', nargs=-1, type=click.Path(exists=True, readable=True, dir_okay=False))
@click.version_option(version=pillow_mbm.__version__)
def convert_mbm(flip, remove, suffix, extension, output, verbose, filenames):
    """Decode Kerbal Space Program MBM files"""

    pairs = path_pairs(filenames, output, suffix, extension)

    with click.progressbar(pairs, show_eta=False, show_pos=True, item_show_func=lambda x: f'{x[0]}->{x[1]}' if x else '') as bar:
        if verbose:
            bar.is_hidden = True

        for inpath, outpath in bar:
            image = Image.open(inpath)

            if flip:
                image = image.transpose(Image.FLIP_TOP_BOTTOM)

            image.save(outpath)

            if remove:
                os.remove(inpath)

            if verbose:
                print(f'Converting: {inpath} -> {outpath}')

    if verbose:
        print(f'Done: converted {len(pairs)} files')


if __name__ == '__main__':
    convert_mbm()



import os.path
from setuptools import setup

project_path = os.path.dirname(os.path.realpath(__file__))

with open(os.path.join(project_path, 'pillow_mbm', 'version.py')) as f:
    exec(f.read())

with open(os.path.join(project_path, 'README.md')) as f:
    readme = f.read()

setup(
    name='pillow-mbm',
    description="A pillow plugin that adds support for KSP's MBM textures",
    version=__version__,
    url='https://github.com/drewcassidy/Pillow-mbm',
    author='Andrew Cassidy',
    long_description=readme,
    long_description_content_type='text/markdown',
    python_requires=">=3.7",
    install_requires=['Pillow', 'click'],
    entry_points={
        'console_scripts': ['convert-mbm = pillow_mbm.__main__:convert_mbm']
    },
    package_dir={'': '.'},
    packages=['pillow_mbm'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Games/Entertainment :: Simulation',
        "Topic :: Multimedia :: Graphics",
        "Topic :: Multimedia :: Graphics :: Graphics Conversion",
    ],
)
